@@ -26,124 +26,6 @@ static CalBackendWCAPClass *parent_class = NULL;
 
 G_DEFINE_TYPE (CalBackendWCAPEvents, cal_backend_wcap_events,  CAL_BACKEND_WCAP_TYPE)
 
-static gboolean
-compare_dates (ECalComponentDateTime *adt, ECalComponentDateTime *bdt)
-{
-	if (!adt->value && !bdt->value)
-		return TRUE;
-	else if (!adt->value || !bdt->value)
-		return FALSE;
-
-	if ((adt->tzid && !bdt->tzid)
-	    || (!adt->tzid && bdt->tzid))
-		return FALSE;
-	
-	if (adt->tzid && bdt->tzid && strcmp (adt->tzid, bdt->tzid))
-		return FALSE;
-	
-	return !icaltime_compare_date_only (*adt->value, *bdt->value);
-}
-
-static gboolean
-compare_times (ECalComponentDateTime *adt, ECalComponentDateTime *bdt)
-{
-	if (!adt->value && !bdt->value)
-		return TRUE;
-	else if (!adt->value || !bdt->value)
-		return FALSE;
-
-	if ((adt->tzid && !bdt->tzid)
-	    || (!adt->tzid && bdt->tzid))
-		return FALSE;
-	
-	if (adt->tzid && bdt->tzid && strcmp (adt->tzid, bdt->tzid))
-		return FALSE;
-	
-	return (adt->value->hour == bdt->value->hour) 
-		&& (adt->value->minute == bdt->value->minute) 
-		&& (adt->value->second == bdt->value->second);
-}
-
-static gboolean
-start_equal (ECalComponent *acomp, ECalComponent *bcomp, gboolean times)
-{
-	ECalComponentDateTime adt, bdt;
-	gboolean retval;
-	
-	e_cal_component_get_dtstart (acomp, &adt);
-	e_cal_component_get_dtstart (bcomp, &bdt);
-	
-	retval = times ? compare_times (&adt, &bdt) : compare_dates (&adt, &bdt);
-	
-	e_cal_component_free_datetime (&adt);
-	e_cal_component_free_datetime (&bdt);
-
-	return retval;
-}
-
-static gboolean
-end_equal (ECalComponent *acomp, ECalComponent *bcomp, gboolean times)
-{
-	ECalComponentDateTime adt, bdt;
-	gboolean retval;
-	
-	e_cal_component_get_dtend (acomp, &adt);
-	e_cal_component_get_dtend (bcomp, &bdt);
-	
-	retval = times ? compare_times (&adt, &bdt) : compare_dates (&adt, &bdt);
-	
-	e_cal_component_free_datetime (&adt);
-	e_cal_component_free_datetime (&bdt);
-
-	return retval;
-}
-
-static gboolean
-rrules_equal (ECalComponent *acomp, ECalComponent *bcomp)
-{
-	GSList *alist, *blist, *l, *m;
-	gboolean retval = TRUE;
-	
-	/* If neither have rrules, we call them equal */
-	if (!e_cal_component_has_rrules (acomp) && !e_cal_component_has_rrules (bcomp))
-		return TRUE;
-
-	/* If only one has rrules, they are not equal */
-	if ((e_cal_component_has_rrules (acomp) && !e_cal_component_has_rrules (bcomp))
-	    || (!e_cal_component_has_rrules (acomp) && e_cal_component_has_rrules (bcomp)))
-		return FALSE;
-
-	/* Try to match each rrule */
-	e_cal_component_get_rrule_list (acomp, &alist);
-	e_cal_component_get_rrule_list (bcomp, &blist);
-
-	for (l = alist; l != NULL; l = l->next) {
-		struct icalrecurrencetype *a = l->data;
-		gboolean match = FALSE;
-		
-		for (m = blist; m != NULL; m = m->next) {
-			struct icalrecurrencetype *b = m->data;
-			
-			if (!memcmp (a, b, sizeof (struct icalrecurrencetype))) {
-				blist = g_slist_remove_link (blist, m);
-				e_cal_component_free_recur_list (m);
-				match = TRUE;
-				break;
-			}
-		}
-		
-		if (!match) {
-			retval = FALSE;
-			break;
-		}
-	}
-
-	e_cal_component_free_recur_list (alist);
-	e_cal_component_free_recur_list (blist);
-	
-	return retval;
-}
-
 static ECalComponent *
 find_master_event (CalBackendWCAP *wcap, ECalComponent *comp)
 {
@@ -480,11 +362,7 @@ process_events_component_to_server (CalBackendWCAP *wcap, ECalComponent *comp, g
 
 				trigger.type = E_CAL_COMPONENT_ALARM_TRIGGER_ABSOLUTE;
 				is_neg = trigger.u.rel_duration.is_neg;
-				icaltime_adjust (&t, is_neg ? -(trigger.u.rel_duration.days + (7 * trigger.u.rel_duration.weeks))
-						 : (trigger.u.rel_duration.days + (7 * trigger.u.rel_duration.weeks)),
-						 is_neg ? -trigger.u.rel_duration.hours : trigger.u.rel_duration.hours,
-						 is_neg ? -trigger.u.rel_duration.minutes : trigger.u.rel_duration.minutes,
-						 is_neg ? -trigger.u.rel_duration.seconds : trigger.u.rel_duration.seconds);
+				t = icaltime_add(t, trigger.u.rel_duration);
 				trigger.u.abs_time = t;
 				trigger.u.abs_time.is_utc = 1;
 
@@ -544,163 +422,9 @@ process_events_component_to_server (CalBackendWCAP *wcap, ECalComponent *comp, g
 	e_cal_component_abort_sequence (comp);
 }
 
-static ECalBackendSyncStatus
-handle_import (CalBackendWCAP *wcap, ECalComponent *comp, CalObjModType mod, 
-	       icalcomponent *toplevel_comp, icalcomponent *subcomp, icalcomponent_kind child_kind) 
-{
-	ECalComponent *existing;
-	icalcomponent *toplevel_import, *import_comp;
-	icalcompiter iter;
-	icalcomponent_kind import_kind;
-	const char *uid;
-	guint error_code;
-	ECalBackendSyncStatus result = GNOME_Evolution_Calendar_Success;
-
-	e_cal_component_get_uid (comp, &uid);
-							
-	existing = g_hash_table_lookup (cal_backend_wcap_get_objects (wcap), uid);
-	
-	/* If its the original item being saved again, it won't have
-	 * the mangled uid so we'll get the first instance */
-	if (!existing && e_cal_component_has_recurrences (comp) && !e_cal_component_is_instance (comp)) {
-		GList *ilist;
-		
-		ilist = g_hash_table_lookup (cal_backend_wcap_get_instances (wcap), uid);
-		if (ilist)
-			existing = ilist->data;
-	}
-	
-	/* If it already exists in our hashes, unmangle the uid so we
-	   import it with the correct uid */
-	if (existing && e_cal_component_is_instance (comp))
-		sunone_util_unmangle_uid (comp);
-	e_cal_component_get_uid (comp, &uid);
-
-	toplevel_import = e_cal_util_new_top_level ();
-				
-	iter = icalcomponent_begin_component (toplevel_comp, ICAL_ANY_COMPONENT);
-	while ((import_comp = icalcompiter_deref (&iter))) {
-		import_kind = icalcomponent_isa (import_comp);
-		if (import_kind != child_kind && import_kind != ICAL_VTIMEZONE_COMPONENT) {
-			icalcompiter_next (&iter);
-			continue;
-		}
-					
-		if (import_kind == ICAL_VTIMEZONE_COMPONENT) {
-			icalcomponent *clone;
-						
-			clone = icalcomponent_new_clone (import_comp);
-			icalcomponent_add_component (toplevel_import, clone);
-		} else if (import_comp == subcomp) {
-			icalcomponent *clone;
-			icalproperty *prop, *prop_clone;
-
-			if (!existing || !e_cal_component_has_recurrences (comp) || mod == CALOBJ_MOD_THIS) {
-				clone = icalcomponent_new_clone (import_comp);
-				icalcomponent_add_component (toplevel_import, clone);
-			} else {
-				GList *ilist, *l;
-				gpointer orig_key;
-				gboolean time_change;
-
-				if (!g_hash_table_lookup_extended (cal_backend_wcap_get_instances (wcap), uid, &orig_key, (gpointer *)&ilist))
-					g_assert_not_reached ();
-
-				if (!start_equal (comp, existing, FALSE) || !end_equal (comp, existing, FALSE) || !rrules_equal (comp, existing)) {
-					/* Delete the old ones and let the server re-expand */
-					sunone_connection_deleteevents_by_id (cal_backend_wcap_get_connection (wcap), cal_backend_wcap_get_calid (wcap), uid, cal_backend_wcap_get_rid_string (existing),
-									      cal_backend_wcap_to_sunone_mod (mod, existing));
-
-					clone = icalcomponent_new_clone (import_comp);
-					prop = icalcomponent_get_first_property (clone, ICAL_RECURRENCEID_PROPERTY);
-					icalcomponent_remove_property (clone, prop);
-					icalcomponent_add_component (toplevel_import, clone);
-
-					icalcompiter_next (&iter);
-					continue;
-				}
-
-				time_change = !start_equal (comp, existing, TRUE) || !end_equal (comp, existing, TRUE);
-
-				switch (mod) {
-				case CALOBJ_MOD_THISANDPRIOR:
-					break;
-				case CALOBJ_MOD_THISANDFUTURE:
-					ilist = g_list_find_custom (ilist, comp, cal_backend_wcap_instance_list_compare);
-					break;
-				case CALOBJ_MOD_ALL:
-				default:
-					break;
-				}
-
-				for (l = ilist; l; l = l->next) {
-					ECalComponent *recur_comp = l->data;
-					icalcomponent *recur_icalcomp;
-					ECalComponentDateTime dt;
-					struct icaltimetype tt;
-									
-					recur_icalcomp = e_cal_component_get_icalcomponent (recur_comp);
-					clone = icalcomponent_new_clone (import_comp);
-
-					/* Set correct recurrence id */
-					prop = icalcomponent_get_first_property (recur_icalcomp, ICAL_RECURRENCEID_PROPERTY);
-					prop_clone = icalproperty_new_clone (prop);
-									
-					prop = icalcomponent_get_first_property (clone, ICAL_RECURRENCEID_PROPERTY);
-					icalcomponent_remove_property (clone, prop);
-					icalproperty_free (prop);
-					icalcomponent_add_property (clone, prop_clone);
-						
-					/* Preserve the start date (and possibly the time) */
-					e_cal_component_get_dtstart (comp, &dt);
-					prop = icalcomponent_get_first_property (recur_icalcomp, ICAL_DTSTART_PROPERTY);
-					tt = icalproperty_get_dtstart (prop);
-						
-					if (time_change) {
-						/* FIXME handle the case where an instance has a different tz */
-						tt.hour = dt.value->hour;
-						tt.minute = dt.value->minute;
-						tt.second = dt.value->second;
-					}
-					prop = icalcomponent_get_first_property (clone, ICAL_DTSTART_PROPERTY);
-					icalproperty_set_dtstart (prop, tt);
-					e_cal_component_free_datetime (&dt);
-
-					/* Preserve the end date (and possibly the time) */
-					e_cal_component_get_dtend (comp, &dt);
-					prop = icalcomponent_get_first_property (recur_icalcomp, ICAL_DTEND_PROPERTY);
-					tt = icalproperty_get_dtend (prop);
-						
-					if (time_change) {
-						/* FIXME handle the case where an instance has a different tz */
-						tt.hour = dt.value->hour;
-						tt.minute = dt.value->minute;
-						tt.second = dt.value->second;
-					}
-					prop = icalcomponent_get_first_property (clone, ICAL_DTEND_PROPERTY);
-					icalproperty_set_dtend (prop, tt);
-					e_cal_component_free_datetime (&dt);
-
-					icalcomponent_add_component (toplevel_import, clone);
-				}
-			}
-		}
-					
-		icalcompiter_next (&iter);
-	}
-				
-	error_code = sunone_connection_import (cal_backend_wcap_get_connection (wcap), cal_backend_wcap_get_calid (wcap), toplevel_import);
-	if (!SUNONE_ERROR_IS_SUCCESSFUL (error_code))
-		result = cal_backend_wcap_result_from_error (error_code);
-				
-	icalcomponent_free (toplevel_import);
-
-	return result;
-}
-
-static ECalBackendSyncStatus
+ECalBackendSyncStatus
 cal_backend_wcap_events_update_objects (ECalBackendSync *backend, EDataCal *cal,
-				 const char *calobj, CalObjModType mod, char **old_object)
+				 const char *calobj, CalObjModType mod, char **old_object, char **new_object)
 {
 	ECalBackendSyncStatus result = GNOME_Evolution_Calendar_Success;
 	CalBackendWCAP *wcap = CAL_BACKEND_WCAP (backend);
@@ -714,6 +438,8 @@ cal_backend_wcap_events_update_objects (ECalBackendSync *backend, EDataCal *cal,
 
 	if (!cal_backend_wcap_is_online (wcap))
 		return GNOME_Evolution_Calendar_RepositoryOffline;
+	if (!cal_backend_wcap_get_connection (wcap))
+		return cal_backend_wcap_get_open_error (wcap);
 
 	toplevel_comp = icalparser_parse_string (calobj);
 	if (!toplevel_comp)
@@ -722,10 +448,24 @@ cal_backend_wcap_events_update_objects (ECalBackendSync *backend, EDataCal *cal,
 	kind = icalcomponent_isa (toplevel_comp);
 	if (kind == ICAL_VEVENT_COMPONENT) {
 		icalcomponent *icalcomp;
+		icalproperty *prop;
 	
 		/* Create a temporary toplevel component and put the VEVENT
 		   in it, to simplify the code below. */
 		icalcomp = toplevel_comp;
+		prop = icalcomponent_get_first_property (icalcomp, ICAL_X_PROPERTY);
+		while (prop) {
+			const char *x_name;
+
+			x_name = icalproperty_get_x_name (prop);
+
+			if (!strcmp (x_name, "X-NSCP-GSE-COMPONENT-STATE")) {
+				icalcomponent_remove_property (icalcomp, prop);
+				break;			
+			}
+
+			prop = icalcomponent_get_next_property (icalcomp, ICAL_X_PROPERTY);
+		}
 		toplevel_comp = e_cal_util_new_top_level ();
 		icalcomponent_add_component (toplevel_comp, icalcomp);
 	} else if (kind != ICAL_VCALENDAR_COMPONENT) {
@@ -744,10 +484,8 @@ cal_backend_wcap_events_update_objects (ECalBackendSync *backend, EDataCal *cal,
 			icalcomponent *return_icalcomp, *return_subcomp;
 			const char *uid;
 			gboolean allday, expand;
-			ECalComponentOrganizer organizer;
 			guint error_code;			
 			SunOneMethod method;
-			gboolean needImport = FALSE;
 			
 			e_cal_component_set_icalcomponent (comp, subcomp);
 
@@ -757,39 +495,8 @@ cal_backend_wcap_events_update_objects (ECalBackendSync *backend, EDataCal *cal,
 			/* Fix up the component for the server */
 			process_events_component_to_server (wcap, comp, &relativeAlarm, &allday);
 
-			e_cal_component_get_organizer (comp, &organizer);
-			if (organizer.value) {
-				if (!strncasecmp ("mailto:", organizer.value, 7)) {
-					/* If we received this via email, we need to import it to the server */
-					needImport = TRUE;
-				} else if (strcasecmp (organizer.value, cal_backend_wcap_get_calid (wcap))) {
-					/* If we are not the organizer and we are not in the attendee list,
-					 * it must be a import */
-					GSList *attendees, *l;
-
-					if (!e_cal_component_has_attendees (comp)) {
-						needImport = TRUE;
-					} else
-					{
-						e_cal_component_get_attendee_list (comp, &attendees);
-						needImport = TRUE;
-						for (l = attendees; l!= NULL; l=l->next)
-						{
-							ECalComponentAttendee *a = l->data;
-							if (!strcasecmp (cal_backend_wcap_get_calid (wcap), a->value))
-							{
-								needImport = FALSE;
-								break;
-							}
-						}
-					}
-
-				}
-
-			}
-
-			if (needImport) {
-				result = handle_import (wcap, comp, mod, toplevel_comp, subcomp, child_kind);
+			if (cal_backend_wcap_need_import (wcap, comp)) {
+				result = cal_backend_wcap_handle_import (wcap, comp, mod, toplevel_comp, subcomp, child_kind);
 				subcomp = icalcomponent_get_next_component (toplevel_comp, ICAL_ANY_COMPONENT);
 				hasImport = TRUE;
 				continue;
@@ -830,7 +537,7 @@ cal_backend_wcap_events_update_objects (ECalBackendSync *backend, EDataCal *cal,
 			return_subcomp = icalcomponent_get_first_component (return_icalcomp,
 									    child_kind);
 			while (return_subcomp) {
-				cal_backend_wcap_add_component (wcap, return_subcomp, TRUE, old_object);
+				cal_backend_wcap_add_component (wcap, return_subcomp, TRUE, old_object, new_object);
 				return_subcomp = icalcomponent_get_next_component (return_icalcomp,
 										   child_kind);
 			}
@@ -847,6 +554,8 @@ cal_backend_wcap_events_update_objects (ECalBackendSync *backend, EDataCal *cal,
 
 	if (hasImport || relativeAlarm)
 		cal_backend_wcap_poll_cb (wcap);
+	else
+		cal_backend_wcap_write_cache (wcap);
 
 	return result;
 }
@@ -861,7 +570,7 @@ cal_backend_wcap_events_create_object (ECalBackendSync *backend, EDataCal *cal,
 
 	d(printf("cal_backend_wcap_events_create_object(%p, %s)\n", backend, *calobj));
 
-	result = cal_backend_wcap_events_update_objects (backend, cal, *calobj, mod, NULL);
+	result = cal_backend_wcap_events_update_objects (backend, cal, *calobj, mod, NULL, NULL);
 /*
 	*calobj = NULL;
 */
@@ -879,7 +588,7 @@ cal_backend_wcap_events_modify_object (ECalBackendSync *backend, EDataCal *cal,
 
 	d(printf("cal_backend_wcap_events_modify_object(%p, %s)\n", backend, calobj));
 
-	result = cal_backend_wcap_events_update_objects (backend, cal, calobj, mod, old_object);
+	result = cal_backend_wcap_events_update_objects (backend, cal, calobj, mod, old_object, new_object);
 
 	return result;
 }
@@ -900,6 +609,8 @@ cal_backend_wcap_events_remove_object (ECalBackendSync *backend, EDataCal *cal,
 
 	if (!cal_backend_wcap_is_online (wcap))
 		return GNOME_Evolution_Calendar_RepositoryOffline;
+	if (!cal_backend_wcap_get_connection (wcap))
+		return cal_backend_wcap_get_open_error (wcap);
 
 	status = E_CAL_BACKEND_SYNC_CLASS (parent_class)->get_object_sync (backend, cal, uid, rid, &calobj);
 	if (status != GNOME_Evolution_Calendar_Success)
@@ -929,7 +640,7 @@ cal_backend_wcap_events_remove_object (ECalBackendSync *backend, EDataCal *cal,
 		return cal_backend_wcap_result_from_error (error_code);
 	}
 
-	cal_backend_wcap_remove_component (wcap, uid, mod);
+	cal_backend_wcap_remove_component (wcap, uid, mod, old_object);
 	g_free (calobj);
 
 	cal_backend_wcap_write_cache (wcap);
@@ -945,6 +656,9 @@ get_fb_info (SunOneConnection *so_cnc, const char *user, const char *calid, time
 	guint error_code;
 	const char *user_calid;
 
+	if (!so_cnc)
+		return; 
+
 	utc_zone = icaltimezone_get_utc_timezone ();
 
 	if (strchr (user, '@')) {
@@ -1004,6 +718,8 @@ cal_backend_wcap_events_get_freebusy (ECalBackendSync *backend, EDataCal *cal, G
 
 	if (!cal_backend_wcap_is_online (wcap))
 		return GNOME_Evolution_Calendar_RepositoryOffline;
+	if (!cal_backend_wcap_get_connection (wcap))
+		return cal_backend_wcap_get_open_error (wcap);
 
 	if (!users) {
 		get_fb_info (cal_backend_wcap_get_connection (wcap), cal_backend_wcap_get_calid (wcap), cal_backend_wcap_get_calid (wcap), start, end, freebusy);
@@ -1032,6 +748,8 @@ cal_backend_wcap_events_receive_objects (ECalBackendSync *backend, EDataCal *cal
 
 	if (!cal_backend_wcap_is_online (wcap))
 		return GNOME_Evolution_Calendar_RepositoryOffline;
+	if (!cal_backend_wcap_get_connection (wcap))
+		return cal_backend_wcap_get_open_error (wcap);
 
 	toplevel_comp = icalparser_parse_string (calobj);
 	if (!toplevel_comp)
