@@ -1229,7 +1229,7 @@ get_cache_name (CalBackendWCAP *wcap, gboolean create)
 	if (!dirname)
 		return NULL;
 		
-	if (create && e_util_mkdir_hier (dirname, S_IRWXU) != 0) {
+	if (create && g_mkdir_with_parents (dirname, S_IRWXU) != 0) {
 		g_free (dirname);
 		return NULL;
 	}
@@ -2353,7 +2353,7 @@ cal_backend_wcap_get_changes (ECalBackendSync *backend, EDataCal *cal, const cha
 	
 	g_object_unref (G_OBJECT (account));
 
-	if (e_util_mkdir_hier (dirname, S_IRWXU) != 0) {
+	if (g_mkdir_with_parents (dirname, S_IRWXU) != 0) {
 		g_message ("cal_backend_wcap_get_changes: Cannot create directory %s", dirname);
 		return GNOME_Evolution_Calendar_OtherError;
 	}
